/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.assignJan18;

import java.util.LinkedList;

import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.FleetGlobal;
import com.sunlabs.archsim.gasp.Function;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.TriggerAction;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;
import com.sunlabs.archsim.globals.Globals;

/**
 * @author ib27688
 *
 */
public class Sink extends Component {

	Globals globals= Globals.getInstance();
	
	Terminal in;
	Terminal ackIn;
	
	SinkAction sinkAction;
	
	LinkedList<Payload> dataReceived;		
	
	public Sink(String name) {
		super(name, null);
		
		dataReceived= new LinkedList<Payload>();	

		sinkAction= new SinkAction("FifoSinkAction");
		addMember(sinkAction);
		in= new InputTerminal(FleetGlobal.IN, this);
		ackIn= new OutputTerminal(FleetGlobal.ACK_IN, this);
		//attach the terminals
		sinkAction.attach(in);
		sinkAction.attach(ackIn);
	} // FifoSource
	
	public Terminal getTerminal(String name) {
		if (name.equals(FleetGlobal.IN)) {
			return in;
		}
		else if (name.equals(FleetGlobal.ACK_IN)) {
			return ackIn;
		}
		else {
			// unknown terminal
			return null;
		}
	} // getTerminal
	
	/**
	 * Get results...
	 * @return a list of results of tests
	 */
	LinkedList<Payload> getResults() {
		return dataReceived;
	} // getResults
	
	public String stateInfo() {

		
		return "Sink " + getName() + ", id= " + getID() + ", has received  " 
		+ sinkAction.count
		+ " items: "
		+ dataReceived;
	} // stateInfo
		
	private class SinkAction extends TriggerAction {
		
		// the number or items to stre
		final int maxStored;
		int count= 0;
				
		public SinkAction(String name) {
			super(name);
			
			// start enabled
			enabled= true;
			
			// how many data items to store
			Integer IMax= globals.intValue(Assign1Defaults.SINK_MAX_STORED);
			if (IMax != null && IMax > 0) {
				maxStored= IMax;
			}
			else {
				maxStored= Assign1Defaults.SINK_MAX_STORED_DEF;
			}
			
			outputFunctions.put(FleetGlobal.ACK_IN, new ActionFired());
		} // FifoSinkAction
		
		/** Vacuous function used to acknowldge that the action has fired */ 
		private class ActionFired extends Function {		
			
			private boolean displayReceived;
			
			public ActionFired() {
				super("Vacous");
				Integer fDelay= globals.intValue(Assign1Defaults.SINK_DELAY);
				functionDelay= (fDelay != null)
					? new Delay(fDelay.intValue())
					: Assign1Defaults.SINK_DELAY_DEF;
					
					Boolean dRec= globals.booleanValue(Assign1Defaults.SINK_DISPLAY_RECEIVED);	
					displayReceived= (dRec != null)
						? dRec
						: Assign1Defaults.SINK_DISPLAY_RECEIVED_DEF;					
			}
		
			public Object value() {
				// GetData
				Object d= in.getData();
				if (d instanceof Payload) {
					Payload p= (Payload)d;
					// have we reached the boundary?
					while (dataReceived.size() >= maxStored) {
						dataReceived.removeFirst();
					}
					// store the item
					dataReceived.add(p);
					// increase count
					count++;
					// if count goes out of whack, turn off
					if (count == Integer.MAX_VALUE) {
						enabled= false;
					}
					if (displayReceived) {
						director.info("Sink " 
								+ Sink.this.getName() 
								+ ", ID= "
								+ Sink.this.getID()
								+ ", received payload "
								+ p);
					}
				}
				else {
					director.fatalError("Sink "
							+ Sink.this.getName() 
							+ ", ID= "
							+ Sink.this.getID()
							+ ", received a non-Payload object: " + d);
				}
				return null;
			} // value
		} // AcknowledgeData

	} // class SinkAction
} // class FifoSink


